#
#  Epour - A bittorrent client using EFL and libtorrent
#
#  Copyright 2012-2013 Kai Huuhko <kai.huuhko@gmail.com>
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
#  MA 02110-1301, USA.
#

import os

import libtorrent as lt

import elementary as elm

import Notify

class Preferences(elm.InnerWindow):
    def __init__(self, parent, session):
        self.session = session
        elm.InnerWindow.__init__(self, parent.win)

        self.limit_widgets = {}
        self.num_widgets = {}

        limit_units = ( "kB/s", "MB/s", "GB/s" )

        self.limits = (
            ( "Upload limit", session.upload_rate_limit, session.set_upload_rate_limit, ),
            ( "Download limit", session.download_rate_limit, session.set_download_rate_limit ),
            ( "Upload limit for local connections", session.local_upload_rate_limit, session.set_local_upload_rate_limit ),
            ( "Download limit for local connections", session.local_download_rate_limit, session.set_local_download_rate_limit ),
        )

        mbox = elm.Box(self)
        mbox.size_hint_weight = 1.0, 1.0
        mbox.size_hint_align = -1.0, -1.0

        lbl = elm.Label(self)
        lbl.text = "Click on the names to open/close configuration entry."
        lbl.show()
        mbox.pack_end(lbl)

        for text, rfunc, wfunc in self.limits:
            f = elm.Frame(self)
            f.collapse = True
            f.autocollapse = True
            f.size_hint_align = -1.0, 0.0
            f.text = text

            b = elm.Box(self)
            b.horizontal = True
            b.show()

            s = elm.Spinner(self)
            s.min_max = 0, 1024
            s.show()
            b.pack_end(s)

            hs = elm.Hoversel(self)
            hs.text = limit_units[0]
            for u in limit_units:
                hs.item_add(u, None, 0, lambda x=hs, y=None, u=u: x.text_set(u))
            hs.show()
            b.pack_end(hs)

            l = rfunc()
            if l > 1024*1024*1024:
                hs.text = "GB/s"
                s.value = round(l/(1024*1024*1024))
            elif l > 1024*1024:
                hs.text = "MB/s"
                s.value = round(l/(1024*1024))
            elif l > 1024:
                s.value = round(l/(1024))

            self.limit_widgets[text] = (s, hs)

            f.content = b
            f.show()
            mbox.pack_end(f)

        # Listen port
        f = elm.Frame(self)
        f.size_hint_align = -1.0, -1.0
        f.collapse = True
        f.autocollapse = True
        f.text = "Listen port (range)"

        b = elm.Box(self)
        b.horizontal = True

        l = self.listenlow = elm.Spinner(self)
        l.min_max = 0, 65535
        l.show()
        l.value = session.listen_port()
        b.pack_end(l)

        h = self.listenhigh = elm.Spinner(self)
        h.min_max = 0, 65535
        h.show()
        h.value = session.listen_port()
        b.pack_end(h)

        f.content = b
        f.show()
        mbox.pack_end(f)

        # Proxy configuration
        pf = self.pf = ProxyConfig(self, parent.win)
        pf.show()
        mbox.pack_end(pf)

        # Data storage
        lbl = elm.Label(self)
        lbl.text = "Torrent data storage path"
        self.dlsel = dlsel = elm.FileselectorEntry(self)
        dlsel.size_hint_align = -1.0, 0.0
        dlsel.inwin_mode = True
        dlsel.folder_only = True
        dlsel.text = "Select data storage path"
        dlsel.path = self.session.conf.get("Settings", "storage_path")
        dlsel.show()
        lbl.show()
        mbox.pack_end(dlsel)

        # Ask confirmations
        chk = self.confirm_chk = elm.Check(self)
        chk.text = "Ask for confirmation on exit"
        chk.state = True if self.session.conf.get("Settings", "confirmations") == "True" else False
        chk.show()
        mbox.pack_end(chk)


        btnb = elm.Box(self)
        btnb.horizontal = True

        save = elm.Button(self)
        save.text_set("Save")
        save.callback_clicked_add(self.save_cb)
        save.show()
        btnb.pack_end(save)

        xbtn = elm.Button(self)
        xbtn.text_set("Close")
        xbtn.callback_clicked_add(lambda x: self.delete())
        xbtn.show()
        btnb.pack_end(xbtn)

        btnb.show()
        mbox.pack_end(btnb)

        scr = elm.Scroller(self)
        scr.policy = elm.ELM_SCROLLER_POLICY_OFF, elm.ELM_SCROLLER_POLICY_AUTO
        scr.content = mbox

        self.content = scr
        self.activate()

    def save_cb(self, btn):
        for text, rfunc, wfunc in self.limits:
            s, hs = self.limit_widgets[text]
            v, u = s.value, hs.text
            if u == "kB/s":
                v = int(v*1024)
            elif u == "MB/s":
                v = int(v*1024*1024)
            elif u == "GB/s":
                v = int(v*1024*1024*1024)
            wfunc(v)

        # Save proxy config to session
        self.pf.save_conf()

        if not os.path.exists(self.dlsel.path):
            p = Notify.Error(self, "Invalid storage path", "You have selected an invalid data storage path for torrents.")
            return
        self.session.conf.set("Settings", "storage_path", self.dlsel.path)
        self.session.conf.set("Settings", "confirmations", str(self.confirm_chk.state))
        self.session.save_conf()

        self.session.listen_on(int(self.listenlow.value), int(self.listenhigh.value))
        self.delete()

class ProxyConfig(elm.Frame):
    def __init__(self, parent, canvas):
        elm.Frame.__init__(self, canvas)

        session = self.session = parent.session

        self.widgets = {}

        self.proxy_types = {
            lt.proxy_type.none.name: lt.proxy_type.none,
            lt.proxy_type.socks4.name: lt.proxy_type.socks4,
            lt.proxy_type.socks5.name: lt.proxy_type.socks5,
            lt.proxy_type.socks5_pw.name: lt.proxy_type.socks5_pw,
            lt.proxy_type.http.name: lt.proxy_type.http,
            lt.proxy_type.http_pw.name: lt.proxy_type.http_pw,
        }

        self.proxies = {
            "Proxy for torrent peer connections": (session.peer_proxy, session.set_peer_proxy,),
            "Proxy for torrent web seed connections": (session.web_seed_proxy, session.set_web_seed_proxy,),
            "Proxy for tracker connections": (session.tracker_proxy, session.set_tracker_proxy,),
            "Proxy for DHT connections": (session.dht_proxy, session.set_dht_proxy,),
        }

        self.collapse = True
        self.autocollapse = True
        self.size_hint_weight = (1.0, 0.0)
        self.size_hint_align = (-1.0, 0.0)
        self.text = "Proxy configuration"

        mbox = elm.Box(self)

        for text, funcs in self.proxies.iteritems():
            rfunc, wfunc = funcs
            settings = rfunc()

            widgets = self.widgets[text] = {}

            pf = elm.Frame(self)
            pf.collapse = True
            pf.autocollapse = True
            pf.size_hint_weight = (1.0, 0.0)
            pf.size_hint_align = (-1.0, 0.0)
            pf.text = text

            box = elm.Box(self)
            box.size_hint_weight = (1.0, 0.0)

            f = elm.Frame(self)
            f.size_hint_align = (-1.0, 0.0)
            f.text = "Proxy type"
            ptype = widgets["type"] = elm.Hoversel(self)
            ptype.text = settings.type.name
            for v in self.proxy_types.iterkeys():
                ptype.item_add(v, callback=lambda x, y, z=v: x.text_set(z))
            f.content = ptype
            f.show()
            box.pack_end(f)

            f = elm.Frame(self)
            f.size_hint_align = (-1.0, 0.0)
            f.text = "Hostname"
            phost = widgets["host"] = elm.Entry(self)
            phost.single_line = True
            phost.entry = settings.hostname
            phost.show()
            f.content = phost
            f.show()
            box.pack_end(f)

            f = elm.Frame(self)
            f.size_hint_align = (-1.0, 0.0)
            f.text = "Port"
            pport = widgets["port"] = elm.Spinner(self)
            pport.min_max = (0, 65535)
            pport.value = settings.port
            pport.show()
            f.content = pport
            f.show()
            box.pack_end(f)

            f = elm.Frame(self)
            f.size_hint_align = (-1.0, 0.0)
            f.text = "Username"
            puser = widgets["user"] = elm.Entry(self)
            puser.single_line = True
            puser.entry = settings.username
            puser.show()
            f.content = puser
            f.show()
            box.pack_end(f)

            f = elm.Frame(self)
            f.size_hint_align = (-1.0, 0.0)
            f.text = "Password"
            ppass = widgets["pass"] = elm.Entry(self)
            ppass.single_line = True
            ppass.password = True
            ppass.entry = settings.password
            ppass.show()
            f.content = ppass
            f.show()
            box.pack_end(f)

            box.show()

            pf.content = box
            pf.show()
            mbox.pack_end(pf)

        mbox.show()
        self.content = mbox

    def save_conf(self):
        for k, w in self.widgets.iteritems():
            rfunc, wfunc = self.proxies[k]
            p = lt.proxy_settings()

            p.hostname = w["host"].entry.encode("utf-8")
            p.port = int(w["port"].value)
            p.username = w["user"].entry.encode("utf-8")
            p.password = w["pass"].entry.encode("utf-8")
            p.type = self.proxy_types[w["type"].text]

            wfunc(p)



# TODO:
# max uploads?
# max conns?
# max half open conns?

# ip filter

# encryption:
# enc in policy
# enc out policy
# allowed enc level
# prefer enc
